/* 
 *   FILE: callbacks.c --
 * AUTHOR: Martin Man <mman@martinman.net>
 *   DATE: 28 December 2005
 *
 * Copyright (c) 2005 Martin Man.
 *
 * This application is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this application; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gdk/gdkkeysyms.h>
#include <glib/gprintf.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <libwnck/workspace.h>

#include <string.h> /* for strlen */

#include "src/callbacks.h"
#include "src/iswitchwin-application.h"
#include "src/wnck.h"

static ISwitchWinApplication *
_gtk_widget_get_iswitchwin_application( GtkWidget * widget )
{
    GtkWidget * toplevel = NULL;
    toplevel = gtk_widget_get_toplevel( widget );
    return (ISwitchWinApplication*) g_object_get_data(
            G_OBJECT( toplevel ), "iswitchwin" );
}

gboolean
on_window_delete_event( GtkWidget * widget, GdkEvent * event,
        gpointer user_data )
{
    ISwitchWinApplication * application = NULL;

    application = _gtk_widget_get_iswitchwin_application( widget );

    /* keep our process running, just hide top-level window */
    gtk_widget_hide( GTK_WIDGET( application->toplevel ) );

    return TRUE;
}

void
on_window_destroy( GtkWidget * widget, gpointer user_data )
{
    gtk_main_quit();
}

gboolean
on_window_key_press_event( GtkWidget * widget, GdkEventKey * event,
        gpointer user_data )
{
    ISwitchWinApplication * application = NULL;
    gunichar pressedkey = 0;
    gint needupdate = FALSE;

    application = _gtk_widget_get_iswitchwin_application( widget );
    g_assert( application != NULL );

    pressedkey = gdk_keyval_to_unicode( event->keyval );

    /* handle Esc */
    if( event->keyval == GDK_Escape ) {
        /* keep our process running, just hide top-level window */
        gtk_widget_hide( GTK_WIDGET( application->toplevel ) );
        return TRUE;
    }

    /* handle Tab */
    if( event->keyval == GDK_Tab ) {
        /* move cursor to the next entry / wrap around */
        {
            GtkTreePath * path = NULL;
            gtk_tree_view_get_cursor(
                    GTK_TREE_VIEW( application->treeview ), &path, NULL );

            gtk_tree_path_next( path );
            gtk_tree_view_set_cursor(
                    GTK_TREE_VIEW( application->treeview ),
                    path, NULL, FALSE );
            gtk_tree_view_get_cursor(
                    GTK_TREE_VIEW( application->treeview ), &path, NULL );

            /* wrap around when at the last entry */
            if( path == NULL ) {
                GtkTreePath * newpath = gtk_tree_path_new_first();
                gtk_tree_view_set_cursor(
                        GTK_TREE_VIEW( application->treeview ),
                        newpath, NULL, FALSE );
                g_free( newpath );
            }
        }
    }

    /* handle Backspace */
    if( event->keyval == GDK_BackSpace ) {
        /* TODO: possibly needs to be reworked */
        {
            gchar * result = "";
            result = g_utf8_find_prev_char( application->typeahead,
                    application->typeahead + strlen( application->typeahead ) );
            if( result != NULL ) {
                * result = 0;
                needupdate = TRUE;
            }
        }
    }

    /* handle printable keys */
    if( g_unichar_isprint( pressedkey ) ) {
        needupdate = TRUE;

        /* TODO: possibly needs to be reworked */
        {
            gchar formated[ 5 ] = "";
            gchar * result = "";
            g_sprintf( formated, "%c", pressedkey );
            result = g_strconcat( application->typeahead, formated, NULL );
            g_free( application->typeahead );
            application->typeahead = result;
        }
    }

    /* update model if necessary, let the treeview handle the other keys */
    if( needupdate ) {
        iswitchwin_application_repaint_window_list( application );
        return TRUE;
    } else {
        return FALSE;
    }
}

void
on_treeview_row_activated( GtkTreeView * treeview,
        GtkTreePath * arg1, GtkTreeViewColumn * arg2, gpointer user_data)
{
    ISwitchWinApplication * application = NULL;
    GtkTreeIter iter;
    WnckWorkspace * wkspc = NULL;
    WnckWindow * w = NULL;
    GTimeVal now;
    guint32 eventtime;

    application = _gtk_widget_get_iswitchwin_application(
            GTK_WIDGET( treeview ) );

    /* get the selected item */
    gtk_tree_model_get_iter( gtk_tree_view_get_model( treeview ),
            &iter, arg1 );
    gtk_tree_model_get( gtk_tree_view_get_model( treeview ),
            &iter, COLUMN_WNCK_WINDOW, &w, -1 );

    /* TODO: eventtime algorithm is wrong, latest metacity prints warnings
     * about incorrect timestamps being sent along with _NET_WM_ACTIVATE, do
     * not know what to do with it */

    /* get current time in miliseconds */
    g_get_current_time( &now );
    eventtime = ( now.tv_sec * 1000 ) + ( now.tv_usec / 1000 );

    /* activate workspace (not for omnipresent windows) & window via wnck*/
    wkspc = wnck_window_get_workspace( w );
    if( wkspc != NULL ) wnck_workspace_activate( wkspc, eventtime );
    wnck_window_activate( w, eventtime );

    /* keep our process running, just hide top-level window */
    gtk_widget_hide( GTK_WIDGET( application->toplevel ) );
}

/* vim: set et ts=4 sw=4 tw=76:
 */
