/* 
 *   FILE: iswitchwin-application.c --
 * AUTHOR: Martin Man <mman@martinman.net>
 *   DATE: 28 December 2005
 *
 * Copyright (c) 2005 Martin Man.
 *
 * This application is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this application; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <libwnck/workspace.h>

#include "src/iswitchwin-application.h"
#include "src/wnck.h"

#include "iswitchwin-application-service-client.h"
#include "iswitchwin-application-service-server.h"

#define MY_DBUS_SERVICE     "com.mman.iswitchwin.ApplicationService"
#define MY_DBUS_PATH        "/com/mman/ISwitchWin"
#define MY_DBUS_INTERFACE   "com.mman.iswitchwin.Application"

static gpointer parent_class = NULL;

static gboolean
_iswitchwin_setup_columns( ISwitchWinApplication * app );

static void
iswitchwin_application_init( ISwitchWinApplication * app )
{
}

static void
iswitchwin_application_class_init( ISwitchWinApplicationClass * app )
{
    /* GObjectClass * gobject_class = G_OBJECT_CLASS( app ); */
    parent_class = g_type_class_ref( G_TYPE_OBJECT );
}

GType
iswitchwin_application_get_type( void )
{
    static GtkType iswitchwin_application_type = 0;

    if( !iswitchwin_application_type )
    {
        static const GTypeInfo iswitchwin_application_info =
        {
            sizeof (ISwitchWinApplicationClass),
            NULL,
            NULL,
            (GClassInitFunc) iswitchwin_application_class_init,
            NULL,
            NULL,
            sizeof (ISwitchWinApplication),
            16,
            (GInstanceInitFunc) iswitchwin_application_init,
        };

        iswitchwin_application_type = g_type_register_static(
                G_TYPE_OBJECT, "ISwitchWinApplication",
                &iswitchwin_application_info, 0 );
    }

    return iswitchwin_application_type;
}

ISwitchWinApplication *
iswitchwin_application_new()
{
    GError * error = NULL;
    ISwitchWinApplication * ret = NULL;

    ret = g_object_new( ISWITCHWIN_TYPE_APPLICATION, NULL );
    ret->typeahead = g_strconcat( "", NULL );
    ret->all_windows = NULL;
    ret->switchable_windows = NULL;

    ret->connection = dbus_g_bus_get( DBUS_BUS_SESSION, &error );
    if( ret->connection == NULL ) {
        g_warning( error->message );
        g_error_free( error );
    }

    return ret;
}

gboolean
iswitchwin_application_build_ui( ISwitchWinApplication * application,
        GladeXML * xml )
{
    g_return_val_if_fail( xml != NULL, FALSE );

    application->xml = xml;

    /* prepare pattern label*/
    application->pattern = glade_xml_get_widget( xml, "pattern" );
    g_assert( GTK_IS_LABEL( application->pattern ) );

    /* prepare model & treeview */
    application->model = gtk_list_store_new( N_COLUMNS,
            GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER );

    application->treeview = glade_xml_get_widget( xml, "treeview" );
    g_assert( GTK_IS_TREE_VIEW( application->treeview ) );

    gtk_tree_view_set_model( GTK_TREE_VIEW( application->treeview ),
            GTK_TREE_MODEL( application->model ) );

    _iswitchwin_setup_columns( application );

    /* prepare toplevel window */
    application->toplevel = glade_xml_get_widget( xml, "window" );
    g_assert( GTK_IS_WINDOW( application->toplevel ) );

    g_object_set_data( G_OBJECT( application->toplevel ), "iswitchwin", application );

    /* connect signals */
    glade_xml_signal_autoconnect( xml );

    return TRUE;
}

/**
 * Create appropriate columns in our treeview
 */
static gboolean
_iswitchwin_setup_columns( ISwitchWinApplication * app )
{
    GtkTreeViewColumn * pix = NULL;
    GtkTreeViewColumn * name = NULL;
    GtkTreeViewColumn * wkspc = NULL;

    g_return_val_if_fail( app != NULL, FALSE );

    pix = gtk_tree_view_column_new_with_attributes(
            "Icon", gtk_cell_renderer_pixbuf_new(),
            "pixbuf", COLUMN_ICON, NULL );
    gtk_tree_view_column_set_sizing( pix, GTK_TREE_VIEW_COLUMN_AUTOSIZE );

    name = gtk_tree_view_column_new_with_attributes(
            "Window", gtk_cell_renderer_text_new(), "text",
            COLUMN_NAME, NULL );
    gtk_tree_view_column_set_sizing( name, GTK_TREE_VIEW_COLUMN_AUTOSIZE );

    wkspc = gtk_tree_view_column_new_with_attributes(
            "Workspace", gtk_cell_renderer_text_new(), "text",
            COLUMN_WORKSPACE, NULL );
    gtk_tree_view_column_set_sizing( wkspc, GTK_TREE_VIEW_COLUMN_AUTOSIZE );

    gtk_tree_view_append_column( GTK_TREE_VIEW( app->treeview ), pix );
    gtk_tree_view_append_column( GTK_TREE_VIEW( app->treeview ), name );
    gtk_tree_view_append_column( GTK_TREE_VIEW( app->treeview ), wkspc );

    return TRUE;
}

gboolean
iswitchwin_application_register_service(
        ISwitchWinApplication * application )
{
    GError * error = NULL;
    DBusGProxy * driver_proxy;
    guint request_name_result;

    g_return_val_if_fail( application != NULL, FALSE );

    driver_proxy = dbus_g_proxy_new_for_name(
            application->connection,
            DBUS_SERVICE_DBUS,
            DBUS_PATH_DBUS,
            DBUS_INTERFACE_DBUS );

    if( !org_freedesktop_DBus_request_name( driver_proxy,
                "com.mman.iswitchwin.ApplicationService",
                0, &request_name_result, &error )) {
        g_warning( error->message );
        g_clear_error( &error );
    }

    if( request_name_result == DBUS_REQUEST_NAME_REPLY_EXISTS ) {
        return FALSE;
    }

    dbus_g_object_type_install_info( ISWITCHWIN_TYPE_APPLICATION,
            &dbus_glib__object_info );
    dbus_g_connection_register_g_object(
            application->connection,
            MY_DBUS_PATH,
            G_OBJECT( application ));

    return TRUE;
}

void
iswitchwin_application_shutdown(
       ISwitchWinApplication * application )
{
    /* TODO: free all allocated resources */
    g_object_unref( application );
    gtk_main_quit();
}

void
iswitchwin_application_open_window(
        ISwitchWinApplication * application,
        guint32 timestamp )
{
    g_return_if_fail( application != NULL );
    g_return_if_fail( GTK_IS_WIDGET( application->toplevel ) );

    /* refresh the list of all windows */
    iswitchwin_application_query_window_list( application );

    /* repaint the list of switchable windows */
    iswitchwin_application_repaint_window_list( application );

    /* bring our window on screen */
    gtk_window_stick( GTK_WINDOW( application->toplevel ) );
    gtk_window_set_keep_above( GTK_WINDOW( application->toplevel ), TRUE );

#if 1 /* TODO: trying to resolve focus issue, this code is stolen from gedit */
    if( !GTK_WIDGET_REALIZED( application->toplevel ) ) {
        gtk_widget_realize( GTK_WIDGET( application->toplevel ) );
    }
#endif
#if 1 /* TODO: trying to resolve focus issue, this code is stolen from gedit */
    {
        guint32 timestamp = gdk_x11_get_server_time(
                GTK_WIDGET( application->toplevel )->window );
        gdk_x11_window_set_user_time(
                GTK_WIDGET( application->toplevel )->window, timestamp );
    }
#endif

    gtk_window_present_with_time(
            GTK_WINDOW( application->toplevel ),
            GDK_CURRENT_TIME );
}

void
iswitchwin_application_open_remote_window(
        ISwitchWinApplication * application )
{
    GError * error = NULL;
    DBusGProxy * remote_object = NULL;

    g_return_if_fail( application != NULL );

    /* get proxy for remote object */
    remote_object = dbus_g_proxy_new_for_name(
            application->connection,
            MY_DBUS_SERVICE,
            MY_DBUS_PATH,
            MY_DBUS_INTERFACE );

    /* singnal remote object to open its window */
    if( !com_mman_iswitchwin_Application_open_window(
                remote_object, 0, &error ) ) {
        g_warning( error->message );
        g_clear_error( &error );
    }
}

void
iswitchwin_application_query_window_list(
        ISwitchWinApplication * application )
{
    WnckScreen * screen = NULL;

    g_assert( application != NULL );

    /* reset typeahead */
    g_free( application->typeahead );
    application->typeahead = g_strconcat( "", NULL );

    /* refresh list of all windows */
    g_list_free( application->all_windows );
    application->all_windows = NULL;
    g_list_free( application->switchable_windows );
    application->switchable_windows = NULL;

    screen = wnck_screen_get_default();
    wnck_screen_force_update( screen );
    application->all_windows = window_list_get( screen,
            WNCK_WINDOW_STATE_SKIP_PAGER | WNCK_WINDOW_STATE_SKIP_TASKLIST );

    /* list of switchable windows will be refreshed by calling
     * repaint_window_list */
}

void
iswitchwin_application_repaint_window_list(
        ISwitchWinApplication * application )
{
    g_assert( application != NULL );

    /* update feedback for the user */
    gtk_label_set_text( GTK_LABEL( application->pattern ),
            g_strconcat( "Showing windows matching: '",
                application->typeahead, "'", NULL ) );

    /* recreate list of switchable windows */
    g_list_free( application->switchable_windows );
    application->switchable_windows = window_list_filter(
            application->all_windows, application->typeahead );

    /* update our treeview model */
    gtk_list_store_clear( application->model );
    {
        GtkTreeIter iter;
        WnckWindow * w = NULL;
        WnckWorkspace * wkspc = NULL;
        const gchar * wname = NULL;
        const gchar * wkspcname = NULL;
        GdkPixbuf * wicon = NULL;

        GList * i = application->switchable_windows;

        for( ; i != NULL; i = g_list_next( i ) ) {
            gtk_list_store_append( application->model, &iter );
            w = (WnckWindow*) i->data;
            wname = wnck_window_get_name( w );
            wicon = wnck_window_get_mini_icon( w );
            wkspc = wnck_window_get_workspace( w );
            if( wkspc != NULL ) {
                wkspcname = wnck_workspace_get_name( wkspc );
            } else {
                wkspcname = "";
            }

            gtk_list_store_set( application->model, &iter,
                    COLUMN_ICON, wicon,
                    COLUMN_NAME, wname,
                    COLUMN_WORKSPACE, wkspcname,
                    COLUMN_WNCK_WINDOW, w,
                    -1 );
        }

    }

    /* select first entry in treeview */
    {
        GtkTreePath * path = gtk_tree_path_new_first();
        gtk_tree_view_set_cursor(
                GTK_TREE_VIEW( application->treeview ),
                path, NULL, FALSE );
        g_free( path );
    }
}

/* vim: set et ts=4 sw=4 tw=76:
 */
